import logging
logger = logging.getLogger('piou.config.dmol')
from piou import config

class DMol(config.AtomConfigGenerator):
    inpfile=None
    def __init__(self,coord_file):
        self.inpfile=coord_file

    def export_atomic_configuration(self,atomic_coordinates,to_file=None,frame_no=None,all_frames=True):
        frameno = len(atomic_coordinates)-1
        if not all_frames and frame_no>=0 and frame_no<frameno:
            frameno = frame_no
        file = open(to_file,mode='w')
        for icoord in range(len(atomic_coordinates)):
            if not all_frames and icoord != frameno:
                continue
            coord = atomic_coordinates[icoord]
            unitcell = coord.get_unit_cell()
            if coord.is_cell_ready():
                (avec,bvec,cvec) = unitcell.get_lattice_vector(to_unit=config.BOHR)
                file.write('$cell vectors\n')
                file.write('    '+str(avec[0])+' '+str(avec[1])+' '+str(avec[2])+'\n')
                file.write('    '+str(bvec[0])+' '+str(bvec[1])+' '+str(bvec[2])+'\n')
                file.write('    '+str(cvec[0])+' '+str(cvec[1])+' '+str(cvec[2])+'\n')
            file.write('$coordinates\n')
            for atom in coord:
                pos = atom.get_pos(mode=config.CARTESIAN, to_unit=config.BOHR)
                elem = atom.get_element_name()
                file.write(elem+" "+str(pos[0])+" "+str(pos[1])+" "+str(pos[2])+'\n')
            file.flush()
            file.close()

    def _gen_atomic_configuration(self):
        file=open(self.inpfile)
        incell=False
        incoords=False
        cellvec=[]
        atms = []
        for line in file:
            line=line.strip()
            words=line.split()
            if line.startswith('$cell vectors'):
                incell=True
                continue
            if line.startswith('$coordinates'):
                incoords=True
            if incell and not incoords and len(words)>=3:
                try:
                    vec = (float(words[0]),float(words[1]),float(words[2]))
                    cellvec.append(vec)
                    continue
                except ValueError:
                    logger.error('invalid cell : '+line)
                    continue
            if incoords and len(words)>=4:
                try:
                    elem=words[0]
                    rx = float(words[1])
                    ry = float(words[2])
                    rz = float(words[3])
                    atms.append((elem,rx,ry,rz))
                except ValueError:
                    logger.error('invalid atomic coordinate : '+line)
                    continue
        conf = config.AtomConfig(coordinate_system=config.CARTESIAN, length_unit=config.BOHR, \
        a_vector=cellvec[0],b_vector=cellvec[1],c_vector=cellvec[2])
        for atm in atms:
            elem=atm[0]
            rx=atm[1]
            ry=atm[2]
            rz=atm[3]
            conf.add_atom(element=elem,rx=rx,ry=ry,rz=rz)

        return [conf]

    def get_name(self):
        return 'DMol3 format'

    def get_defaultname(self,args):
        return 'coord.dmol'




import logging
import os

from os.path import join
from os.path import expanduser
from piou.util import pyutil

logger = logging.getLogger("piou.data.properties")

class Property:
    __metaclass__ = pyutil.Singleton
    props = {}
    propfile_from_arg=''
    def __init__(self):
        pass

    def set_propfile_from_args(self,p):
        self.propfile_from_arg=p

    def set_property(self,property_name,val):
        ''' set the specified property '''
        if len(self.props.keys())==0:
            self.load_props()
            logger.debug("properties defined:")
            for k,v in self.props.iteritems():
                logger.debug(" "+k+" = "+v)
        self.props[property_name] = val

    def get_property(self,property_name,valtype="string"):
        ''' obtain the specified property. the properties can be defined in :
           ~./chase/chase.ini, ~/.chase/acvconfig or ~/.piou

            the method will try to convert the obtained string according to 'valtype'.
            'valtype' can be one of None or 'string' (no conversion), 'float', 'int', or 'bool'
            if conversion fails, will return None for 'float' and 'int', False for 'bool'.  '''
        if len(self.props.keys())==0:
            self.load_props()
            logger.debug("properties defined:")
            for k,v in self.props.iteritems():
                logger.debug(" "+k+" = "+v)
        if property_name in self.props:
            if valtype is None or valtype=='string':
                return self.props[property_name]
            elif valtype == 'float' or valtype == float:
                try:
                    return float(self.props[property_name])
                except ValueError:
                    logger.error("property : ["+property_name+" = "+\
                                 self.props[property_name]+"] is not a float")
                    return None
            elif valtype == 'int' or valtype == int:
                try:
                    return int(self.props[property_name])
                except ValueError:
                    logger.error("property : ["+property_name+" = "+\
                                 self.props[property_name]+"] is not an int")
                    return None
            elif valtype == 'bool' or valtype==bool:
                return pyutil.parse_bool(self.props[property_name])
            
        return None

    def load_props(self):
        dir = join(pyutil.map_package_to_dir('piou'),'data')
        propfile = dir+"/props.properties"
        self.load_props_sub(propfile)
    
        homedir = expanduser("~")
        self.load_props_sub(homedir+"/.chase/chase.ini")
        self.load_props_sub(homedir+"/.chase/acvconfig")
        self.load_props_sub(homedir+"/.piou")
        if len(self.propfile_from_arg)!=0:
            self.load_props_sub(self.propfile_from_arg)

    def load_props_sub(self,filename):
        if not os.path.exists(filename):
            logger.debug("property file : "+filename+" does not exist.")
            return
        f = open(filename,'r')
        for line in f:
            line = line.strip()
            if line.startswith("#"):
                continue
            words=line.split('=')
            if len(words)>=2:
                self.props[words[0].strip()] = words[1].strip()
        f.close()





import logging
logger=logging.getLogger('piou')
import sys

if sys.hexversion >= 0x02030000: #Python 2.3 and over
    import logging
    import optparse
else:
    from piou.ext import logging
    from piou.ext import optparse
import os

import piou.data.properties

def initialize_logger_and_opts(pioutil):
    parser = optparse.OptionParser(\
    usage="%prog [options]",\
    description=\
    "The PHASE I/O utility script."+ "\n"+\
    "This script will enable you to :\n"+\
    " 1. validate your PHASE input, \n"+\
    " 2. generate default input file from atomic coordinates and \n"+\
    " 3. convert PHASE input/output data for use with other programs, and vice-versa.\n"+\
    "refer to the user's manual for details.",\
    version="%prog 0.5")

    opts = get_opts(parser,pioutil)
    init_logger(opts.loglevel)
    if opts.propfile is not None:
        piou.data.properties.Property().set_propfile_from_args(opts.propfile)
    if opts.ppdir is not None:
        piou.data.properties.Property().set_property("pp.default_pp_dir",opts.ppdir)
    logger.debug("pp directory : "+piou.data.properties.Property().get_property("pp.default_pp_dir"))

    return opts

def get_opts(parser,pioutil):
    parser.add_option("-l", "--loglevel", type="choice",choices=("0","1","2"),\
                      dest="loglevel", help="specify the log level (one of 0,1,2)",default="1")
    parser.add_option("", "--prop",dest="propfile",help=\
   "specify the property file. this will overload the default settings",default=None)
    parser.add_option("--ppdir",dest="ppdir",help="specify the directory where the pp files reside."\
    +" if unspecified, the string specified in the property file will be used.",default=None)
    parser.add_option("-b","--batch",action="store_true",\
                      help="specify this option in order to perform this script in batch mode (for geninp and conv). ",\
                      dest="batch",default=False)

    pioutil.fill_opts(parser)

    (options, args) = parser.parse_args()
    return options

logformat  = '%(levelname)10s: %(message)s'
logformat_dbg = '%(asctime)8s %(name)20s %(lineno)6d %(levelname)10s: %(message)s'
logtimefmt = '%a %d %b %H:%M:%S'
logger  = logging.getLogger("main")

def init_logger(loglevel):
    if loglevel=="2":
        logging.basicConfig(level=logging.DEBUG,
                    format=logformat_dbg,
                    datefmt=logtimefmt
                    )
    elif loglevel=="1":
        logging.basicConfig(level=logging.INFO,
                    format=logformat,
                    datefmt=logtimefmt
                    )
    else:
        logging.basicConfig(level=logging.WARN,
                    format=logformat,
                    datefmt=logtimefmt
                    )

def run_main(pioutil):
    inst = pioutil()
    print inst.get_description()
    print 'Copyright (C) the RISS project, The University of Tokyo'
    opts=initialize_logger_and_opts(inst)
    inst.initialize(opts)
    inst.run()

class PHASE_IO_utility:
    ''' the base class for the main script of the various phase io utility '''

    def __init__(self):
        pass

    def get_name(self):
        ''' return the name of this utility '''
        raise NotImplementedError()

    def get_description(self):
        ''' return the description for this utility'''
        raise NotImplementedError()

    def fill_opts(self,parser):
        ''' fill the optsions specific for this phase io utility '''
        raise NotImplementedError()

    def initialize(self,opts):
        ''' initialize using the specified options '''
        raise NotImplementedError()

    def run(self):
        ''' run the main script'''
        raise NotImplementedError()




''' manipulate the XSF (XCrysDen Structure File) format file.
    supports only the 'CRYSTAL' section for the time being... '''

import logging
logger = logging.getLogger('piou.config.xsf')

from piou import config
from piou.data import elements

class XSF(config.AtomConfigGenerator):
    xsffile=None
    def get_defaultname(self,args):
        return 'coord.xsf'

    def get_name(self):
        return 'XSF (XCrysDen Structure File) format'

    def __init__(self,coord_file):
        #if coord_file is None:
        #    self.set_is_valid(False)
        #    logger.error('invalid file')
        #    return
        self.xsffile = coord_file

    def supports_volumetric_data(self):
        return True

    def supports_frame(self):
        return True

    def _gen_atomic_configuration(self):
        frame = []
        f=open(self.xsffile,mode='r')
        tag_crystal='CRYSTAL'
        tag_primvec='PRIMVEC'
        tag_convvec='CONVVEC'
        tag_primcoord='PRIMCOORD'
        tag_atoms='ATOMS'
        tag_animsteps='ANIMSTEPS'
        tag_begin_grid='BEGIN_BLOCK_DATAGRID'
        tag_datagrid='DATAGRID_3D'
        tag_begin_datagrid='BEGIN_DATAGRID_3D'
        tag_end_datagrid='END_DATAGRID_3D'
        tag_end_grid='END_BLOCK_DATAGRID'

        in_crystal=False
        in_primvec=False
        in_primcoord=False
        in_atom=False
        in_anim=False
        in_datagrid=False
        in_datagrid_data=False
        nanim=-1
        avec=None
        bvec=None
        cvec=None
        currconf=None
        nat=-1

        voldata=None
        vdata=None
        ndiv=None
        div=[]
        origin=None
        for line in f:
            line=line.strip()
            words=line.split()

            if line.startswith('#'):
                continue

            if len(line)==0:
                continue

            if words[0].startswith(tag_begin_grid):
                in_datagrid=True
                in_datagrid_data=False
                continue
            
            if in_datagrid and \
                (words[0].startswith(tag_begin_datagrid) or words[0].startswith(tag_datagrid)):
                in_datagrid_data=True
                continue

            if in_datagrid_data and words[0].startswith(tag_end_datagrid):
                in_datagrid_data=False
                if voldata is None:
                    voldata=[]
                avec=(div[2][0],div[2][1],div[2][2])
                bvec=(div[1][0],div[1][1],div[1][2])
                cvec=(div[0][0],div[0][1],div[0][2])
                unitcell = config.UnitCell(a_vector=avec,b_vector=bvec,c_vector=cvec)
                unitcell.set_length_unit(config.ANGSTROM)
                vvdata = config.VolumetricData(unitcell=unitcell,n1=ndiv[2],n2=ndiv[1],n3=ndiv[0],vdata=vdata)
                if origin is not None:
                    vvdata.set_origin(origin)
                currconf.add_volumetric_data(vvdata)
                voldata.append(vvdata)
                vdata=[]
                continue

            if in_datagrid and words[0].startswith(tag_end_grid):
                in_datagrid=False
                div=[]
                ndiv = None
                origin = None
                continue

            if in_datagrid and not in_datagrid_data:
                continue

            if in_datagrid_data and ndiv is None and len(words)>=3:
                try:
                    ndiv = []
                    ndiv.append(int(words[0]))
                    ndiv.append(int(words[1]))
                    ndiv.append(int(words[2]))
                    continue
                except ValueError:
                    logger.error('invalid data grid')
                    continue
             
            if in_datagrid_data and origin is None and len(words)>=3:
                origin=[]
                origin.append(float(words[0]))
                origin.append(float(words[1]))
                origin.append(float(words[2]))
                continue

            if in_datagrid_data and len(div)<3 and ndiv is not None and len(ndiv)==3 and len(words)>=3:
                try:
                    tmp=[]
                    tmp.append(float(words[0]))
                    tmp.append(float(words[1]))
                    tmp.append(float(words[2]))
                    div.append(tmp)
                    continue
                except ValueError:
                    logger.error('invalid spanning vector')
                    continue

            if in_datagrid_data and ndiv is not None and len(ndiv)==3 and len(div)==3:
                if vdata is None:
                    vdata=[]
                try:
                    for word in words:
                        vdata.append(float(word))
                except ValueError:
                    logger.error('non-float value found in grid data')
                    continue

            if in_datagrid:
                continue

            if words[0]==tag_primvec:
                in_primvec=True
                in_primcoord=False
                in_atom=False
                continue

            if words[0]==tag_animsteps:
                in_anim=True
                in_atom=False
                if len(words)>1:
                    try:
                        nanim=int(words[1])
                    except ValueError:
                        pass

            if words[0]==tag_primcoord:
                in_primcoord=True
                in_atom=False
                nat=-1
                continue

            if words[0].startswith('BEGIN'):
                in_atom=False
                in_primcoord=False
                continue

            if words[0]==tag_atoms:
                in_atom=True
                currconf=config.AtomConfig(coordinate_system=config.CARTESIAN,\
                length_unit=config.ANGSTROM)
                frame.append(currconf)
                continue

            if in_atom:
                if len(words)<4:
                    continue
                try:
                    elem = self._resolve_elem(words[0])
                    rx = float(words[1])
                    ry = float(words[2])
                    rz = float(words[3])
                    currconf.add_atom(element=elem,rx=rx,ry=ry,rz=rz)
                    if currconf.get_num_atom()==nat:
                        nat=-1
                    continue
                except ValueError:
                    logger.error('the atom line : '+line+' is not valid')
                    return None

            if in_primcoord:
                if words[0].startswith(tag_crystal):
                    continue
                if nat<0:
                    try:
                        nat = int(words[0])
                    except ValueError:
                        logger.error('the number of atoms is not given properly')
                        return None
                    currconf=config.AtomConfig(coordinate_system=config.CARTESIAN,\
                    length_unit=config.ANGSTROM,a_vector=avec,b_vector=bvec,c_vector=cvec)
                    frame.append(currconf)
                    continue
                if len(words)<4:
                    logger.error(line)
                    logger.error('the primcoord specification must have atleast 4 words')
                    return None
                try:
                    elem = self._resolve_elem(words[0])
                    rx = float(words[1])
                    ry = float(words[2])
                    rz = float(words[3])
                    currconf.add_atom(element=elem,rx=rx,ry=ry,rz=rz)
                    if currconf.get_num_atom()==nat:
                        nat=-1
                    continue
                except ValueError:
                    logger.error('the primcoord line : '+line+' is not valid')
                    return None

            if in_primvec:
                if avec is not None and bvec is not None and cvec is not None:
                    avec=None
                    bvec=None
                    cvec=None
                if avec is None:
                    try:
                        avec=[float(words[0]),float(words[1]),float(words[2])]
                    except IndexError,ValueError:
                        logger.error('invalid primvec specification : '+line)
                        return None
                elif bvec is None:
                    try:
                        bvec=[float(words[0]),float(words[1]),float(words[2])]
                    except IndexError,ValueError:
                        logger.error('invalid primvec specification : '+line)
                        return None
                elif cvec is None:
                    try:
                        cvec=[float(words[0]),float(words[1]),float(words[2])]
                    except IndexError,ValueError:
                        logger.error('invalid primvec specification : '+line)
                        return None
                if avec is not None and bvec is not None and cvec is not None:
                    in_primvec=False
                    continue

        logger.info('the number of frames in this XSF file : '+str(len(frame)))

        if voldata is not None:
            for fra in frame:
                for vol in voldata:
                    fra.add_volumetric_data(vol)
        return frame

    def _resolve_elem(self,elem):
        try:
            ielem = int(elem)
            return elements.ElementInfo().get_element_name_from_atomic_number(ielem)
        except ValueError:
            return str(elem)

    def export_atomic_configuration(self,atomic_coordinates,to_file=None,frame_no=None,all_frames=True):
        file=open(to_file,mode='w')
        nfra = len(atomic_coordinates)
        if nfra>1:
            file.write('ANIMSTEPS '+str(nfra)+'\n')
        for ifra in range(len(atomic_coordinates)):
            frameid=''
            if nfra>1:
                frameid+= ' '+str(ifra+1)
            atomcoord = atomic_coordinates[ifra]
            vdata = atomcoord.get_volumetric_data()
            if vdata is None:
                if atomcoord.is_cell_ready():
                    file.write('CRYSTAL\n')
                    primvec = 'PRIMVEC'+frameid
                    file.write(primvec+'\n')
                    (avec,bvec,cvec) = atomcoord.get_unit_cell().get_lattice_vector(to_unit=config.ANGSTROM)
                    file.write(str(avec[0])+' '+str(avec[1])+' '+str(avec[2])+'\n')
                    file.write(str(bvec[0])+' '+str(bvec[1])+' '+str(bvec[2])+'\n')
                    file.write(str(cvec[0])+' '+str(cvec[1])+' '+str(cvec[2])+'\n')
                    primcoord='PRIMCOORD'+frameid
                    file.write(primcoord+'\n')
                    file.write(str(atomcoord.get_num_atom())+' 1\n')
                else:
                    atom = 'ATOMS'+frameid
                    file.write(atom+'\n')
                    #file.write(str(atomcoord.get_num_atom())+' \n')
                for atom in atomcoord:
                    elem = atom.get_element_name()
                    pos = atom.get_pos(mode=config.CARTESIAN,to_unit=config.ANGSTROM)
                    file.write(elem+' '+str(pos[0])+' '+str(pos[1])+' '+str(pos[2])+'\n')
            else:
                atom = 'ATOMS'+frameid
                file.write(atom+'\n')
                for atom in atomcoord:
                    elem = atom.get_element_name()
                    pos = atom.get_pos(mode=config.CARTESIAN,to_unit=config.ANGSTROM)
                    file.write(elem+' '+str(pos[0])+' '+str(pos[1])+' '+str(pos[2])+'\n')
                for vdat in vdata:
                    if not vdat.has_enough_data():
                        logger.error('not a valid grid data')
                        continue
                    file.write('BEGIN_BLOCK_DATAGRID_3D\n')
                    file.write('3D_grid_data\n')
                    file.write('DATAGRID_3D_G98CUBE\n')

                    ndiv=vdat.get_ndiv()
                    (avec,bvec,cvec) = atomcoord.get_unit_cell().get_lattice_vector(to_unit=config.ANGSTROM)
                    strndiv = str(int(ndiv[2]))+' '+str(int(ndiv[1]))+' '+str(int(ndiv[0]))
                    file.write(strndiv+'\n')
                    orig = vdat.get_origin()
                    file.write(str(orig[0])+' '+str(orig[1])+' '+str(orig[2])+'\n')
                    file.write(str(cvec[0])+' '+str(cvec[1])+' '+str(cvec[2])+'\n')
                    file.write(str(bvec[0])+' '+str(bvec[1])+' '+str(bvec[2])+'\n')
                    file.write(str(avec[0])+' '+str(avec[1])+' '+str(avec[2])+'\n')
                    vvdat = vdat.get_vdata()
                    count = 0
                    nrow=6
                    for v in vvdat:
                        file.write(str(v)+' ')
                        count += 1
                        if count%nrow==0:
                            file.write('\n')
                            count=0
                    file.write('\n')

                    file.write('END_DATAGRID_3D\n')
                    file.write('END_BLOCK_DATAGRID_3D\n')
        file.flush()
        file.close()




"""optik

A powerful, extensible, and easy-to-use command-line parser for Python.

By Greg Ward <gward@python.net>

See http://optik.sourceforge.net/
"""

# Copyright (c) 2001-2003 Gregory P. Ward.  All rights reserved.
# See the README.txt distributed with Optik for licensing terms.

__revision__ = "$Id: __init__.py,v 1.19 2003/04/21 01:53:28 gward Exp $"

__version__ = "1.4.1"


# Re-import these for convenience
from optik.option import Option
from optik.option_parser import *
from optik.help import *
from optik.errors import *


# Some day, there might be many Option classes.  As of Optik 1.3, the
# preferred way to instantiate Options is indirectly, via make_option(),
# which will become a factory function when there are many Option
# classes.
make_option = Option




from piou.util import phase_util
from piou.util import vector_op
from piou.data import properties

import piou.data.elements
import piou.data.constants

from piou import config

import logging
import os
import math

logger = logging.getLogger("piou.config.functions")

def check_cpumax(args):
    fact = properties.Property().get_property("inpcheck.cpumax_warn",float)
    if fact is None:
        return True

    cpum = args['currelem']
    if cpum is None:
        return True
    c = config.phase.get_value_in_default_units(cpum)
    if c <fact:
        logger.warn("[control.cpumax] = "+str(cpum.get_value())+ " sec ... are you sure?")
        args['input'].increment_nwarn()
    return True

def check_tetrahedral_dos(args):
    logger.debug("checking if tetrahedral dos is propertly configured...")
    root = args['input'].get_root_entry()
    kmethod = root.get_entry("accuracy.ksampling.method")
    bok=True

    if kmethod is None:
        logger.warn("[accuracy.ksampling.method] must be defined and its value must be 'mesh' "+\
                    "in order to calculate the dos by the tetrahedron method")
        args['input'].increment_nwarn()
    elif kmethod.get_value() !="mesh":
            bok=False
            logger.warn("[accuracy.ksampling.method] must be 'mesh'"+\
                        " in order to calculate the dos by the tetrahedron method")
            args['input'].increment_nwarn()
    else:
        bok=True

    smear = root.get_entry("accuracy.smearing.method")
    if smear==None:
        logger.warn(\
       "[accuracy.smearing.method] must be defined and its value must be 'tetrahedral' "+\
       "in order to calculate the dos by the tetrahedral method")
        args['input'].increment_nwarn()
        bok=False
    else:
        comp = smear.get_value().strip().lower()
        if not (comp == "tetrahedral" or comp == "tetrahedron" or \
                comp == "improved_tetrahedron" or comp == "t"):
            logger.warn( \
            "[accuracy.smearing.method] must be set to 'tetrahedral' in order to calculate "+\
            "the dos by the tetrahedral method")
            args['input'].increment_nwarn()
            bok=False

    if not bok:
        logger.warn("tetrahedron method is not configured propertly; the gaussain broadening method will be used")
    else:
        logger.debug("tetrahedron method seems to be configured properly.")

    return bok

def check_cutoff_cd(args):
    cutoff_cd = args['currelem']
    cutoff_wf = args['input'].get_root_entry().get_entry('accuracy.cutoff_wf')
    wf = config.phase.get_value_in_default_units(cutoff_wf)
    bo=True
    if wf is None:
        logger.error("invalid specification : "+str(cutoff_wf))
        args['input'].increment_nerr()
        bo=False
    cd = config.phase.get_value_in_default_units(cutoff_cd)
    if cd is None:
        logger.error("invalid specification : "+str(cutoff_cd))
        args['input'].increment_nerr()
        bo=False
    if not bo:
        return False
   
    ok = cd >= 4.0 * wf

    if not ok:
        logger.error("")
        logger.error("the cutoff energy for the charge density : "+\
                      str(cutoff_cd).strip()+" [hartree] is not enough.")
        args['input'].increment_nerr()
    return ok

def check_consistency_of_wavefunction_solver(args):
    ''' check consistency of solver. for instance, 'prec' must be 'off' for the davidson solver. '''
    ok = True
    wfsolvers = args['currelem']
    rmmblock = args['input'].get_root_entry().get_entry('wavefunction_solver.rmm')
    if rmmblock == None:
        rmmblock = args['input'].get_root_entry().get_entry('wf_solver.rmm')
    nsolver = wfsolvers.get_num_data()
    minus_found=False
    maxcmix=1
    maxid=0
    for i in range(nsolver):
        if minus_found:
            logger.warn("found a solver defined after a solver whose 'till_n' attribute is smaller than 0.")
            logger.warn("this is not an error, but mind that such solver will never be used.")
            logger.warn('')
            args['input'].increment_nwarn()

        tilln = wfsolvers.get_data(i,'till_n')
        ii = i+1
        if wfsolvers.get_data(i,'id') is not None:
            ii = wfsolvers.get_data(i,'id')
        elif wfsolvers.get_data(i,'no') is not None:
            ii = wfsolvers.get_data(i,'no')
        if ii==1 and tilln is not None and tilln==1:
            mat = args['input'].get_root_entry().get_entry('accuracy.initial_wavefunctions')
            if mat is not None and mat.get_value() == 'matrix_diagon':
                logger.warn(\
                "till_n for the first solver should be greater than 1 when using 'matrix_diagon' "+\
                "for the initial WF generation")
                logger.warn('')
                args['input'].increment_nwarn()

        sol = wfsolvers.get_data(i,'sol')
        if sol!=None and sol.lower()=='davidson':
            prec = wfsolvers.get_data(i,'prec')
            if prec:
                logger.error("prec must be 'off' when using the davidson solver")
                args['input'].increment_nerr()
                ok=False
            submat = wfsolvers.get_data(i,'submat')
            if submat:
                logger.error("submat must be 'off' when using the davidson solver")
                args['input'].increment_nerr()
                ok=False
            if not ok:
                logger.error("")

        if sol is not None and sol.lower().startswith('rmm'):
            if (rmmblock is None or rmmblock.get_entry('edelta_change_to_rmm') is None):
                logger.warn("'edelta_change_to_rmm' variable not found.")
                logger.warn("it is recommended to specify this variable when using the rmm solver.")
                logger.warn('')
                args['input'].increment_nwarn()
            submat = wfsolvers.get_data(i,'submat')
            if not submat:
                logger.warn("subspace rotation is 'off' ")
                logger.warn("it is strongly recommended to use subspace rotation when using the rmm solver.")
                logger.warn('')

        if tilln is not None and tilln<0:
            minus_found = True

        cmix=wfsolvers.get_data(i,'cmix')
        if cmix is not None and cmix!='*' and cmix>maxcmix:
            maxcmix = cmix
            maxid=i

    cmixtable = args['input'].get_root_entry().get_entry('charge_mixing.mixing_methods.table')
    if cmixtable is not None:
        n = cmixtable.get_num_data()
        nn = n
        for i in range(n):
            nnn = cmixtable.get_data(i,'no')
            if nnn is not None and nnn>nn:
                nn = nnn
            nnn = cmixtable.get_data(i,'id')
            if nnn is not None and nnn>nn:
                nn = nnn
        if nn<maxcmix:
            logger.error("'cmix' for solver no. "+str(maxid+1) +' is invalid : '+str(maxcmix))
            args['input'].increment_nerr()
            ok=False
    elif cmixtable is None and maxcmix >1:
        logger.error('max. cmix ('+str(maxcmix)+') is too large' )
        args['input'].increment_nerr()
        ok=False
            
    return ok

def check_charge_mixing(args):
    cmix_table = args['currelem']
    ncmix = cmix_table.get_num_data()
    ok=True
    for i in range(ncmix):
        nbmix = cmix_table.get_data(i,'nbmix')
        if nbmix is not None and nbmix <3:
            logger.warn("'nbmix' for charge mixing scheme no. "+str(i+1)+' is too small : '+str(nbmix))
            args['input'].increment_nwarn()
            ok=False
    return ok

def check_xctype(args):
    xcfrompp = phase_util.get_xctype_from_pp(args['input'])
    if xcfrompp is None:
        logger.error("XC type defined in the pp file is invalid")
        args['input'].increment_nerr()
        return False

    xctype = args['input'].get_root_entry().get_entry('accuracy.xctype')

    if xctype is None:
        return True

    if xctype.get_value().lower() != xcfrompp.lower():
        if xctype.get_value().lower()=='ggapbex' and xcfrompp.lower() == 'ggapbe': # exceptional case
            return True
        logger.error("the XC type defined in F_INP ["+xctype.get_value() +\
                     "] is inconsistent with that defined in the pp file ["+xcfrompp+"]")
        args['input'].increment_nerr()
        return False

    return True

def check_num_bands(args):
    fact = properties.Property().get_property("inpcheck.band_tolerance_factor",float)
    if fact is None:
        return True

    nbdefined = args['input'].get_root_entry().get_entry('accuracy.num_bands')
    if nbdefined is None:
        logger.error('number of bands is not defined')
        args['input'].increment_nerr()
        return False

    nv = phase_util.get_num_valence_electrons(args['input'])
    if nv is None:
        logger.error("failed to get the number of valence electrons")
        args['input'].increment_nerr()
        return False

    nbmin = nv * 0.5

    if nbdefined.get_value() < nbmin:
        logger.error("the number of bands is insufficient. defined : "+\
                     str(nbdefined.get_value())+", minimum : "+str(int(nbmin))+\
                    ", recommended : "+str(int(nbmin*fact)))
        args['input'].increment_nerr()
        return False

    if int(nbdefined.get_value()) < int(nbmin*fact):
        logger.warn("the number of bands may be insufficient. defined : "+\
                     str(nbdefined.get_value())+", minimum : "+str(int(nbmin))+\
                    ", recommended : "+str(int(nbmin*fact)))
        args['input'].increment_nwarn()

    return True

def all_elements_are_defined(args):
    ok=True

    input = args['input']
    elem_table = args['currelem']
    atom_table = args['input'].get_root_entry().get_entry('structure.atom_list.atoms.table')

    if atom_table is None:
        logger.error('atom table is undefined.')
        args['input'].increment_nerr()
        return False

    nat = atom_table.get_num_data()
    elem_dict={}
    for i in range(nat):
        elem = atom_table.get_data(i,'element')
        if not elem in elem_dict:
            elem_dict[elem] = False

    nelem = elem_table.get_num_data()
    for i in range(nelem):
        elemname = elem_table.get_data(i,'element')
        if elemname is None:
            logger.error("element name undefined")
            args['input'].increment_nerr()
            return False

        if elemname in elem_dict.keys():
            elem_dict[elemname] = True

    for k,v in elem_dict.iteritems():
        if not v and k is not None:
            logger.error("element : '"+k+"' undefined in ["+elem_table.get_long_name()+"]")
            args['input'].increment_nerr()
            ok=False

    return ok

def projector_exists(args):
    proj0 = args['input'].get_root_entry().get_entry('accuracy.hubbard.projectors.table')
    proj1 = args['input'].get_root_entry().get_entry('accuracy.projector_list.projectors.table')
    if proj0 is None or proj1 is None:
        return True
    n0 = proj0.get_num_data()
    n1 = proj1.get_num_data()
    ok=True
    for i in range(n0):
        ii = proj0.get_data(i,'no')
        if ii is None:
            continue
        found = False
        for j in range(n1):
            jj = proj1.get_data(j,'group')
            if jj is None:
                continue
            if jj==ii:
                found=True
                break
        if not found:
            logger.error("projector : "+str(ii)+" specified in accuracy.hubbard.projectors.table "+\
                         "is undefined under accuracy.hubbard.projector_list.projectors.table.")
            args['input'].increment_nerr()
            ok=False
    return ok

def valid_proj_group_assigned(args):
    projs = phase_util.get_atom_attributes(args['input'],'proj_group')
    projtable = args['input'].get_root_entry().get_entry('accuracy.projector_list.projectors.table')
    group = phase_util.get_attributes(projtable,'group')
    if projs is None or group is None:
        return True
    ok = True
    for i in range(len(projs)):
        proj = projs[i]
        if not proj in group:
            if not (proj=='*' or proj==0):
                logger.error("undefined proj_group : "+str(proj)+" assigned to atom "+str(i+1))
                args['input'].increment_nerr()
                ok=False
    return ok

def check_proj_group(args):
    projtable = args['input'].get_root_entry().get_entry('accuracy.projector_list.projectors.table')
    if projtable is None:
        logger.error("projectors undefined.")
        return False
    group = phase_util.get_attributes(projtable,'group')
    l = phase_util.get_attributes(projtable,'l')
    if group is None or l is None:
        return True
            
    for i in range(len(l)):
        for j in range(len(l)):
            if j>=i:
                continue
            if l[i]==l[j] and group[i]==group[j]:
                logger.error("the 'l' attribute for projector "+str(i+1)+" and "+str(j+1)+" are the same "+\
                "and belong to the same group.")
                args['input'].increment_nerr()
                return False
    return True

def check_num_layer(args):
    naxis = 1
    axi = args['input'].get_root_entry().get_entry('postprocessing.ldos.layerdos.normal_axis')
    if axi is not None:
        naxis = axi.get_value()

    nlayer = phase_util.get_atom_attributes(args['input'],'num_layer')
    if nlayer is None:
        return False

    xyz = 'rx'
    if naxis==2:
        xyz = 'ry'
    if naxis==3:
        xyz = 'rz'
    coord = phase_util.get_atom_attributes(args['input'],xyz)
    if coord is None:
        return False

    nlayer_coord={}
    for layer in nlayer:
        nlayer_coord[layer]=[]
    for i in range(len(nlayer)):
        nlayer_coord[nlayer[i]].append(coord[i])

    min = {}
    max = {}
    for k,v in nlayer_coord.iteritems():
        minv=10000000
        maxv=-10000000
        for vv in v:
            if vv<minv:
                minv=vv
            if vv>maxv:
                maxv=vv
        min[k] = minv
        max[k] = maxv

    b=True
    for k,v in nlayer_coord.iteritems():
        for k0,v0 in nlayer_coord.iteritems():
            if k<=k0:
                continue
            if not (min[k0]>max[k] or max[k0] <min[k]):
                logger.error("inconsistent num_layer attribute : "+str(k0)+", "+str(k))
                args['input'].increment_nerr()
                b=False
    return b

def check_mass(args):
    fact = properties.Property().get_property("inpcheck.mass_tolerance_factor",float)
    if fact is None:
        return True

    elemtable = args['input'].get_root_entry().get_entry('structure.element_list.table')
    elemblock = args['input'].get_root_entry().get_entry('structure.element_list')
    if elemtable is None:
        return True
    uni = elemblock.get_default_unit('mass')
    nelem = elemtable.get_num_data()
    for i in range(nelem):
        elemname = elemtable.get_data(i,'element')
        if elemname is None:
            logger.error("element name undefined.")
            return True
        mass = elemtable.get_data(i,'mass')
        if mass is None:
            return True

        if mass is None or mass == '*':
            mass = 51577.50

        if uni == 'atomic_mass':
            mass *= piou.data.elements.ElementInfo().to_au_mass

        compmass = piou.data.elements.ElementInfo().get_element_attribute(elemname,"mass")
        if compmass is None:
            logger.error("invalid element name : "+elemname)
            args['input'].increment_nerr()
            return False
        if mass<compmass/fact:
            logger.warn("the mass "+str(mass)+" [au] of element ["+elemname+"] might be too light. "+\
                        "reference mass : "+str(compmass)+" [au].")
            args['input'].increment_nwarn()
        elif mass>compmass*fact:
            logger.warn("the mass "+str(mass)+" [au] of element ["+elemname+"] might be too heavy. "+\
                        "reference mass : "+str(compmass)+" [au].")
            args['input'].increment_nwarn()

    return True

def check_qmass(args):
    taufac = properties.Property().get_property("inpcheck.tau_mass_factor",float)
    if taufac is None:
        return True
    lmass = properties.Property().get_property("inpcheck.lightest_mass",float)
    if lmass is None:
        return True

    thermo_group = phase_util.get_atom_attributes(args['input'],'thermo_group')
    if thermo_group is None:
        logger.warn("the 'thermo_group' attribute is unassinged")
        args['input'].increment_nwarn()
        return True

    thermostats = args['currelem']
    mass = phase_util.get_mass_list(args['input'])
    wei = phase_util.get_atom_attributes(args['input'],'weight')
    nthermo = thermostats.get_num_data()
    for i in range(nthermo):
        id = thermostats.get_data(i,'id')
        if id is None:
            id = thermostats.get_data(i,'no')
        if id is None:
            id = i+1

        natm = len(thermo_group)
        mass_list_now = []
        nat_now = 0
        for j in range(natm):
            if id == thermo_group[j]:
                if wei is not None and wei==2:
                    nat_now += 2
                else:
                    nat_now += 1
                mass_list_now.append(mass[j])
        if len(mass_list_now) == 0:
            logger.error("thermo_group assignment is not valid")
            args['input'].increment_nerr()
            return False
        mass_list_now.sort()
        mass0 = mass_list_now[0]
        logger.debug("number of atoms associated with thermostat "+str(id)+" : "+str(nat_now))
        logger.debug("mass of the lightest element associated with this thermostat: "\
                      +str(mass0))
        logger.debug("mass of the heaviest element associated with this thermostat: "\
                      +str(mass_list_now[len(mass_list_now)-1]))
 
        temperature = thermostats.get_corresponding_data('id',id,'temp')
        if temperature is None:
            temperature = thermostats.get_corresponding_data('no',id,'temp')
        if temperature is None:
            temperature = thermostats.get_data(i,'temp')

        qmass = thermostats.get_corresponding_data('id',id,'qmass')
        if qmass is None:
            qmass = thermostats.get_corresponding_data('no',id,'qmass')
        if qmass is None:
            qmass = thermostats.get_data(i,'qmass')

        if temperature is None or qmass is None:
            logger.error("thermostat "+str(id)+" is invalid.")
            args['input'].increment_nerr()
            continue
        logger.debug("temperature and qmass associate with thermostat "+str(id)+" : "+\
                      str(temperature)+", "+str(qmass))
        
        tau = taufac * math.sqrt(mass0)
        logger.debug("estimated tau : "+str(tau))
#        qmassref = 2.0*3.0*nat_now*piou.data.constants.kbt*temperature*(0.5*tau/math.pi)**2
        qmassref = config.get_qmass_estimate(args['input'].get_atomic_configuration(),taufac,nat_now,temperature)
        logger.debug("reference value for qmass : "+str(qmassref))
        if qmassref>qmass:
            logger.warn("qmass "+str(qmass)+" [au] for thermostat "+str(id)+" may be too light. "+\
                        "reference value : "+str(qmassref)+" [au].")
            args['input'].increment_nwarn()
    return True

def check_dt(args):
    taufac = properties.Property().get_property("inpcheck.tau_mass_factor",float)
    if taufac is None:
        return True

    ndiv = properties.Property().get_property("inpcheck.tau_ndiv",float)
    if ndiv is None:
        return True

    dt = args['input'].get_root_entry().get_entry('structure_evolution.dt')
    if dt is None:
        return True
    fdt = config.phase.get_value_in_default_units(dt)
    aconf = args['input'].get_atomic_configuration()
    if aconf is None:
        return True
    dtref = config.get_dt_estimate(aconf,taufac,ndiv)
    if dtref is None:
        return True
    logger.debug("reference dt : "+str(dtref))
    logger.debug("defined dt   : "+str(fdt))

    if fdt>dtref*1.1:
        logger.warn("structure_evolution.dt may be too large. defined value : "+str(fdt)+\
                    " [au], reference value : "+str(dtref)+" [au].")
        args['input'].increment_nwarn()

    return True

def check_scf_convergence_criteria(args):
    root = args['input'].get_root_entry()
    deinp = config.phase.get_value_in_default_units(\
            root.get_entry("accuracy.scf_convergence.delta_total_energy"))

    if deinp is None:
        deinp = 1e-10

    de = properties.Property().get_property("inpcheck.scf_criteria.scf",float)

    mobile = phase_util.get_atom_attributes(args['input'],'mobile')
    strwarn = "[accuracy.scf_convergence.delta_total_energy] might be too large"

    if mobile is None:
        if de is not None and deinp>de:
            logger.warn(strwarn+" for a static calculation.")
            logger.warn("specified : "+str(deinp)+" [hartree], reference : "+str(de)+" [hartree].")
            args['input'].increment_nwarn()
        return True
    elif mobile is not None and sum(mobile)==0:
        if de is not None and deinp>de:
            logger.warn(strwarn+" for a static calculation.")
            logger.warn("specified : "+str(deinp)+" [hartree], reference : "+str(de)+" [hartree].")
            args['input'].increment_nwarn()
        return True
        
    swphonon = root.get_entry("phonon.sw_phonon")
    swcalfor = root.get_entry("phonon.sw_calc_force")
    if swphonon is not None and swcalfor is not None and \
       swphonon.get_value() and swcalfor.get_value():
        de = properties.Property().get_property("inpcheck.scf_criteria.phonon",float)
        if de is not None and deinp>de:
            logger.warn(strwarn+" for a phonon calculation.")
            logger.warn("specified : "+str(deinp)+" [hartree], reference : "+str(de)+" [hartree].")
            args['input'].increment_nwarn()
        return True

    strevlm = root.get_entry("structure_evolution.method")
    if strevlm is not None and phase_util.is_md(strevlm.get_value()):
        de = properties.Property().get_property("inpcheck.scf_criteria.md",float)
        if de is not None and deinp>de:
            logger.warn(strwarn+" for a molecular-dynamics simulation.")
            logger.warn("specified : "+str(deinp)+" [hartree], reference : "+str(de)+" [hartree].")
            args['input'].increment_nwarn()
        return True

    de = properties.Property().get_property("inpcheck.scf_criteria.stropt",float)
    if de is not None and deinp>de:
        logger.warn(strwarn+" for structural optimization.")
        logger.warn("specified : "+str(deinp)+" [hartree], reference : "+str(de)+" [hartree].")
        args['input'].increment_nwarn()
    return True

def check_mobile(args):
    mobile = phase_util.get_atom_attributes(args['input'],'mobile')
    method = args['input'].get_root_entry().get_entry('structure_evolution.method').get_value()
    if mobile is None or sum(mobile)==0:
        if phase_util.is_md(method):
            logger.warn("the 'mobile' attribute for all atoms are set to 'off', "+\
                        "which is unlikely for a molecular-dynamics simulation.")
            args['input'].increment_nwarn()
        else:
            logger.warn("the 'mobile' attribute for all atoms are set to 'off', "+\
                        "which is unlikely for structural optimization.")
            args['input'].increment_nwarn()
    return True

def check_k_mesh(args):
    method = args['input'].get_root_entry().get_entry('accuracy.ksampling.method')
    kmesh_dk = properties.Property().get_property("inpcheck.kmesh_dk",float)
    if kmesh_dk is None:
        logger.error("invalid property : inpcheck.kmesh_dk")
        return True

    if method is not None and (method.get_value().lower() != 'monk' and\
                               method.get_value().lower() != 'gamma' and\
                               method.get_value().lower() != 'mesh' ):
        return True

    bgamma=False
    if method is not None and method.get_value().lower() == 'gamma':
        bgamma=True
    
    f=True
    n1 = args['input'].get_root_entry().get_entry('accuracy.ksampling.mesh.nx')
    if n1 is None:
        f = False
        n1 = args['input'].get_root_entry().get_entry('accuracy.ksampling.mp_index.n1')
    if n1 is None:
        return True

    if f:
        n2 = args['input'].get_root_entry().get_entry('accuracy.ksampling.mesh.ny')
        n3 = args['input'].get_root_entry().get_entry('accuracy.ksampling.mesh.nz')
    else:
        n2 = args['input'].get_root_entry().get_entry('accuracy.ksampling.mp_index.n2')
        n3 = args['input'].get_root_entry().get_entry('accuracy.ksampling.mp_index.n3')
    if n2 is None or n3 is None and not bgamma:
        return True

    if not bgamma:
        in1=n1.get_value()
        in2=n2.get_value()
        in3=n3.get_value()
        if in1==0 or in2==0 or in3==0:
            logger.error("invalid kmesh")
            return True
    else:
        in1=1
        in2=1
        in3=1

    aconfig = args['input'].get_atomic_configuration()
    if aconfig is None:
        return True

    (rb1,rb2,rb3) = config.get_reference_kmesh(aconfig,kmesh_dk)
    if rb1 is None or rb2 is None or rb3 is None:
        return True

    logger.debug("rb1, rb2, rb3 : "+str(rb1)+", "+str(rb2)+", "+str(rb3))

    if rb1>in1:
        logger.warn("kmesh for the first reciprocal lattice : ["+str(in1)+"] may not be enough. "+\
                    "reference value : ["+str(rb1)+"].")
        args['input'].increment_nwarn()

    if rb2>in2:
        logger.warn("kmesh for the first second lattice     : ["+str(in2)+"] may not be enough. "+\
                    "reference value : ["+str(rb2)+"].")
        args['input'].increment_nwarn()

    if rb3>in3:
        logger.warn("kmesh for the third reciprocal lattice : ["+str(in3)+"] may not be enough. "+\
                    "reference value : ["+str(rb3)+"].")
        args['input'].increment_nwarn()

    return True

def check_bond_length(args):
    conf = args['input'].get_atomic_configuration()
    if conf is None:
        return True
    fact = properties.Property().get_property("inpcheck.bond_length_factor",float)
    if fact is None:
        return True
    bl = conf.get_bondlenth_list(cutoff_factor=fact)
    if bl is None or len(bl)==0:
        return True
    for b in bl:
        logger.warn("bond length between atom "+str(b[1].index()+1)+" ("+b[1].get_element_name()+\
                    ") and "+str(b[2].index()+1)+" ("+b[2].get_element_name()+") : "+\
                    str(str(b[0])+" [bohr] ").rjust(25) +"may be too short.")
        args['input'].increment_nwarn()

    return True

def check_fract(args):
    conf = args['input'].get_atomic_configuration()
    if conf is None:
        return True

    fact = properties.Property().get_property("inpcheck.max_fract_coord",float)
    if fact is None:
        return True

    found=False
    for atom in conf:
        pos = atom.get_pos(mode=config.FRACTIONAL)
        valid = True
        if math.fabs(pos[0]) > fact or math.fabs(pos[1]) > fact or math.fabs(pos[2]) > fact:
            valid = False
        if not valid:
            logger.warn("fractional coordinate ("+str(pos[0])+" "+str(pos[1])+" "+str(pos[2])+\
                        ") of atom "+str(atom.index()+1)+" may not be valid.")
            found=True
            
    if found:
        args['input'].increment_nwarn()

    return True

def check_inversion(args):
    conf = args['input'].get_atomic_configuration()
    if conf.has_inversion_symmetry():
        inv=args['input'].get_root_entry().get_entry('structure.symmetry.sw_inversion')
        if inv is None or not inv.get_value():
            logger.warn(\
            "the system has inversion symmetry, but [structure.symmetry.sw_inversion] is not 'on'")
            args['input'].increment_nwarn()
    else:
        inv=args['input'].get_root_entry().get_entry('structure.symmetry.sw_inversion')
        if inv is not None and inv.get_value():
            logger.error(\
            "the system does not have inversion symmetry, but [structure.symmetry.sw_inversion] is 'on'")
            args['input'].increment_nerr()
            return False

    return True




import textwrap

__revision__ = "$Id: help.py,v 1.4 2003/04/20 14:12:14 gward Exp $"

__all__ = ['HelpFormatter', 'IndentedHelpFormatter', 'TitledHelpFormatter']


class HelpFormatter:

    """
    Abstract base class for formatting option help.  OptionParser
    instances should use one of the HelpFormatter subclasses for
    formatting help; by default IndentedHelpFormatter is used.

    Instance attributes:
      indent_increment : int
        the number of columns to indent per nesting level
      max_help_position : int
        the maximum starting column for option help text
      help_position : int
        the calculated starting column for option help text;
        initially the same as the maximum
      width : int
        total number of columns for output
      level : int
        current indentation level
      current_indent : int
        current indentation level (in columns)
      help_width : int
        number of columns available for option help text (calculated)
    """

    def __init__ (self,
                  indent_increment,
                  max_help_position,
                  width,
                  short_first):
        self.indent_increment = indent_increment
        self.help_position = self.max_help_position = max_help_position
        self.width = width
        self.current_indent = 0
        self.level = 0
        self.help_width = width - max_help_position
        self.short_first = short_first

    def indent (self):
        self.current_indent += self.indent_increment
        self.level += 1

    def dedent (self):
        self.current_indent -= self.indent_increment
        assert self.current_indent >= 0, "Indent decreased below 0."
        self.level -= 1

    def format_usage (self, usage):
        raise NotImplementedError, "subclasses must implement"

    def format_heading (self, heading):
        raise NotImplementedError, "subclasses must implement"

    def format_description (self, description):
        desc_width = self.width - self.current_indent
        indent = " "*self.current_indent
        return textwrap.fill(description, desc_width,
                             initial_indent=indent,
                             subsequent_indent=indent)

    def format_option (self, option):
        # The help for each option consists of two parts:
        #   * the opt strings and metavars
        #     eg. ("-x", or "-fFILENAME, --file=FILENAME")
        #   * the user-supplied help string
        #     eg. ("turn on expert mode", "read data from FILENAME")
        #
        # If possible, we write both of these on the same line:
        #   -x      turn on expert mode
        #
        # But if the opt string list is too long, we put the help
        # string on a second line, indented to the same column it would
        # start in if it fit on the first line.
        #   -fFILENAME, --file=FILENAME
        #           read data from FILENAME
        result = []
        opts = option.option_strings
        opt_width = self.help_position - self.current_indent - 2
        if len(opts) > opt_width:
            opts = "%*s%s\n" % (self.current_indent, "", opts)
            indent_first = self.help_position
        else:                       # start help on same line as opts
            opts = "%*s%-*s  " % (self.current_indent, "", opt_width, opts)
            indent_first = 0
        result.append(opts)
        if option.help:
            help_lines = textwrap.wrap(option.help, self.help_width)
            result.append("%*s%s\n" % (indent_first, "", help_lines[0]))
            result.extend(["%*s%s\n" % (self.help_position, "", line)
                           for line in help_lines[1:]])
        elif opts[-1] != "\n":
            result.append("\n")
        return "".join(result)

    def store_option_strings (self, parser):
        self.indent()
        max_len = 0
        for opt in parser.option_list:
            strings = self.format_option_strings(opt)
            opt.option_strings = strings
            max_len = max(max_len, len(strings) + self.current_indent)
        self.indent()
        for group in parser.option_groups:
            for opt in group.option_list:
                strings = self.format_option_strings(opt)
                opt.option_strings = strings
                max_len = max(max_len, len(strings) + self.current_indent)
        self.dedent()
        self.dedent()
        self.help_position = min(max_len + 2, self.max_help_position)

    def format_option_strings (self, option):
        """Return a comma-separated list of option strings & metavariables."""
        if option.takes_value():
            metavar = option.metavar or option.dest.upper()
            short_opts = [sopt + metavar for sopt in option._short_opts]
            long_opts = [lopt + "=" + metavar for lopt in option._long_opts]
        else:
            short_opts = option._short_opts
            long_opts = option._long_opts

        if self.short_first:
            opts = short_opts + long_opts
        else:
            opts = long_opts + short_opts

        return ", ".join(opts)

class IndentedHelpFormatter (HelpFormatter):
    """Format help with indented section bodies.
    """

    def __init__ (self,
                  indent_increment=2,
                  max_help_position=24,
                  width=80,
                  short_first=1):
        HelpFormatter.__init__(
            self, indent_increment, max_help_position, width, short_first)

    def format_usage (self, usage):
        return "usage: %s\n" % usage

    def format_heading (self, heading):
        return "%*s%s:\n" % (self.current_indent, "", heading)


class TitledHelpFormatter (HelpFormatter):
    """Format help with underlined section headers.
    """

    def __init__ (self,
                  indent_increment=0,
                  max_help_position=24,
                  width=80,
                  short_first=0):
        HelpFormatter.__init__ (
            self, indent_increment, max_help_position, width, short_first)

    def format_usage (self, usage):
        return "%s  %s\n" % (self.format_heading("Usage"), usage)

    def format_heading (self, heading):
        return "%s\n%s\n" % (heading, "=-"[self.level] * len(heading))



''' maipulate I/O files for the VASP code '''
import logging
logger = logging.getLogger('piou.config.vasp')
import os
from piou import config
from piou.data import properties

class VASP_input(config.AtomConfigGenerator):
    inpfile = None
    elems = None

    def get_defaultname(self,args):
        return 'POSCAR'

    def __init__(self,coord_file):
        if coord_file is None:
            self.set_is_valid(False)
            return
        self.inpfile = coord_file

    def get_elem_map(self):
        return self.elems

    def _gen_atomic_configuration(self):
        f=open(self.inpfile,'r')
        line=[]
        for lin in f:
            line.append(lin.strip())
        f.close()

        if len(line)<8:
            logger.error('insufficient number of lines in POSCAR')
            return None

        pot = 'POTCAR'
        dir = os.path.dirname(self.inpfile)
        if len(dir)>0:
            pot = dir+os.sep+'POTCAR'
        f=open(pot,'r')
        key = 'VRHFIN'
        elems=[]
        for li in f:
            li = li.strip()
            if li.startswith(key):
                w0 = li.split('=')
                if w0>=2:
                    elems.append(w0[1].strip().split(':')[0])
        msg = 'elements resolved from the POTCAR file :'
        for ele in elems:
            msg += ' '+ele
        print msg

        factor=1.0
        try:
            factor = float(line[1].split()[0])
        except ValueError:
            logger.error("the 1st line is invalid : "+line[1])
            return None

        try:
            av=line[2].split()
            av[0] = float(av[0])*factor
            av[1] = float(av[1])*factor
            av[2] = float(av[2])*factor
            bv=line[3].split()
            bv[0] = float(bv[0])*factor
            bv[1] = float(bv[1])*factor
            bv[2] = float(bv[2])*factor
            cv=line[4].split()
            cv[0] = float(cv[0])*factor
            cv[1] = float(cv[1])*factor
            cv[2] = float(cv[2])*factor
        except ValueError,IndexError:
            logger.error('failed to read the unit cell')
            return None

        unitcell = config.UnitCell(\
        a_vector=[av[0],av[1],av[2]],\
        b_vector=[bv[0],bv[1],bv[2]],\
        c_vector=[cv[0],cv[1],cv[2]])

        nat_per_elem_line = line[5].split()
        ver5=False
        try:
            iii = int(nat_per_elem_line[0])
        except ValueError:
            ver5=True
        if ver5:
            logger.info("detected POSCAR for VASP ver.5 (and over)")
        else:
            logger.info("detected POSCAR for VASP ver.4")

        nat_per_elem = []
        for i in range(len(nat_per_elem_line)):
            try:
                ii = int(nat_per_elem_line[i])
                nat_per_elem.append(ii)
            except ValueError:
                if not ver5:
                    logger.error('invalid specification for the number of atoms : '+str(line[5]))
                    return None
                else:
                    continue
        if len(nat_per_elem)!=len(elems):
            logger.error('inconsistent number of elements defined in the POSCAR and POTCAR files')
            return None

        nextind=6
        if line[6].lower().startswith('s'):
            nextind=7

        atomcount=0
        for i in range(len(nat_per_elem)):
            atomcount += nat_per_elem[i]
        if atomcount+nextind>len(line):
            logger.error('insufficient number of lines in POSCAR')
            return None

        mode=config.FRACTIONAL
        factatom=1.0
        if line[nextind].lower().startswith('c') or \
            line[nextind].lower().startswith('k'):
            mode=config.CARTESIAN
            factatom=factor
        atomconf=config.AtomConfig(coordinate_system=mode,\
        length_unit=config.ANGSTROM,unit_cell=unitcell)
        unitcell.set_length_unit(config.ANGSTROM)

        self.elems=[]
        atomcount = 0
        for ielem in range(len(nat_per_elem)):
            elem = elems[ielem]
            for i in range(nat_per_elem[ielem]):
                self.elems.append(elem)
                atomcount += 1
                index = atomcount+nextind
                pos = line[index].split()
                for i in range(3):
                    try:
                        pos[i] = float(pos[i])*factatom
                    except ValueError,IndexError:
                        logger.error('failed to read the atomic coordinates from the POSCAR file')
                        return None
                atomconf.add_atom(element=elem,rx=pos[0],ry=pos[1],rz=pos[2])

        return [atomconf]

    def export_atomic_configuration(self,atomic_coordinates,to_file=None,frame_no=None,all_frames=False):
        frameno=config.resolve_frame_no(frame_no,len(atomic_coordinates))
        atomic_coordinate = atomic_coordinates[frameno]
        if not atomic_coordinate.is_cell_ready():
            logger.error('the unit cell is not set')
            return

        file=open(to_file,'w')
        file.write('first line is a comment \n')
        file.write('1\n')
        (avec,bvec,cvec) = atomic_coordinate.get_unit_cell().get_lattice_vector(to_unit=config.ANGSTROM)
        file.write(str(avec[0])+" "+str(avec[1])+" "+str(avec[2])+'\n')
        file.write(str(bvec[0])+" "+str(bvec[1])+" "+str(bvec[2])+'\n')
        file.write(str(cvec[0])+" "+str(cvec[1])+" "+str(cvec[2])+'\n')

        elements = atomic_coordinate.get_elem_list()
        for element in elements:
            file.write(' '+element)
        for element in elements:
            file.write(' '+str(self._get_num_elem(element,atomic_coordinate)))
        file.write('\n')
        file.write('cartesian\n')

        for atom in atomic_coordinate:
            pos = atom.get_pos(mode=config.CARTESIAN, to_unit=config.ANGSTROM)
            file.write(str(pos[0])+" "+str(pos[1])+" "+str(pos[2])+'\n')

        file.flush()
        file.close()

        # ppfile
        ppdir = properties.Property().get_property('pp.vasp_pp_dir')
        if ppdir is not None and os.path.exists(ppdir) and os.path.isdir(ppdir):
            potcar = open('POTCAR','w')
            potfound=0
            for element in elements:
                dirs = os.listdir(ppdir)
                for dir in dirs:
                    if str(dir).startswith(element):
                        pot = open(ppdir+os.sep+dir+os.sep+'POTCAR',mode='r')
                        for l in pot:
                            potcar.write(l)
                        pot.close()
                        potfound += 1
                        break

            if potfound<len(elements):
                logger.warn('failed to resolve pp files for some of the elements')

    def _get_num_elem(self,elemname,coord):
        ret = 0
        for atom in coord:
            if elemname==atom.get_element_name():
                ret+=1
        return ret

    def get_name(self):
        return 'VASP input format'

class VASP_output(config.AtomConfigGenerator):
    def __init__(self,coord_file):
        if coord_file is None:
            self.set_is_valid(False)
            return
        self.inpfile = coord_file

    def supports_frame(self):
        return True

    def supports_export(self):
        return False

    def _gen_atomic_configuration(self):
        dir = os.path.dirname(self.inpfile)
        if len(dir)>0:
            dir = dir+os.sep

        tmp = VASP_input(coord_file=dir+'POSCAR')
        tmplconfig = tmp.get_atomic_configuration()
        cell = tmplconfig.get_unit_cell()
        frame = []
        natm = tmplconfig.get_num_atom()
        f = open(self.inpfile,'r')
        header_count=0
        config_count=0
        elem_map = tmp.get_elem_map()

        for line in f:

            line = line.strip()
            header_count += 1
            if header_count<=5:
                continue

            if config_count==0:
                config_count += 1
                curr_config = config.AtomConfig(\
                coordinate_system=config.FRACTIONAL,length_unit=config.ANGSTROM,\
                unit_cell=cell)
                frame.append(curr_config)
                continue

            pos = line.split()
            if len(pos)<3:
                logger.error('invalid line found in XDATCAR : '+line)
                config_count += 1
                continue

            try:
                curr_config.add_atom(element=elem_map[config_count-1],\
                rx=float(pos[0]),ry=float(pos[1]),rz=float(pos[2]))
            except ValueError, IndexError:
                logger.error('invalid line found in XDATCAR : '+line)
                config_count += 1
                continue

            if config_count==natm:
                config_count = 0
                continue

            config_count += 1

        return frame

    def export_atomic_configuration(self,atomic_coordinates,to_file=None,frame_no=None,all_frames=False):
        pass

    def get_name(self):
        return 'VASP output format'

    def get_defaultname(self,args):
        return 'XDATCAR'




'''define constants for the package 'piou (PHASE I/O utilities) ' '''
import math

Hartree_2_eV = 27.21139615

eV_2_Hartree = 1/Hartree_2_eV
cminv_2_eV      = 1.233984185e-4
cminv_2_Hartree = cminv_2_eV*eV_2_Hartree
cminv_2_THz     = 1.0/33.0
eV_2_cminv      = 1/cminv_2_eV
rad_2_angle     = 180.0/math.pi
angle_2_rad     = math.pi/180.0

Bohr_2_Angstrom = 0.5291772480

Angstrom_2_Bohr = 1/Bohr_2_Angstrom

Angstrom_2_nm   = 0.1
nm_2_Angstrom   = 10.0

very_small_value = 1e-12

name                 = 'name'
has_table            = 'has_table'
choice               = 'choice'
description          = 'description'
num_elements         = 'num_elements'
columns              = 'columns'
entry_type           = 'entry_type'
entry_type_block     = 'block'
entry_type_primitive = 'primitive'
entry_type_table     = 'table'
entry_type_vector    = 'vector'

val_type             = 'val_type'
val_type_float       = 'float'
val_type_int         = 'int'
val_type_string      = 'string'
val_type_bool        = 'bool'
val_type_fract       = 'fract'

val_range            = 'val_range'

unit_type            = 'unit_type'
unit_type_none       = 'none'
unit_type_length     = 'length'
unit_type_energy     = 'energy'
unit_type_time       = 'time'
unit_type_longtime   = 'longtime'
unit_type_velocity   = 'velocity'
unit_type_force      = 'force'
unit_type_pressure   = 'pressure'
unit_type_mass       = 'mass'
unit_type_angle      = 'angle'

unit_spec={
unit_type_none:[''],
unit_type_length:['bohr','angstrom','nm'],
unit_type_energy:['hartree','ev','rydberg'],
unit_type_time:['au_time','fs','ps','ns'],
unit_type_longtime:['sec','min','hour','day'],
unit_type_velocity:['bohr/au_time','bohr/fs','angstrom/fs','angstrom/au_time','nm/fs','nm/au_time'],
unit_type_force:['hartree/bohr','hartree/angstrom','hartree/nm','ev/angstrom','ev/bohr','ev/nm','rydberg/bohr','rydberg/angstrom','rydberg/nm'],
unit_type_pressure:['hartree/bohr3','hartree/angstrom3','hartree/nm3','ev/angstrom3','ev/bohr3','ev/nm3','rydberg/angstrom3','rydberg/bohr3','rydberg/nm3'],
unit_type_mass:['au_mass','atomic_mass'],
unit_type_angle:['degree','radian']
}

unit_conversion = {
 '':1.0

,'bohr':1.0
,'angstrom':0.5291772480
,'nm':0.05291772480

,'hartree':1.0
,'rydberg':2.0
,'ev':27.211396150

,'au_time':1.0
,'fs':2.418884327e-2
,'ps':2.418884327e-5
,'ns':2.418884327e-8

,'sec':1.0
,'min':1.0/60.0
,'hour':1.0/3600.0
,'day':1.0/86400.0

,'au_mass':9.1093897e-31
,'atomic_mass':1.66053e-27

}

au_mass=9.1093897e-31
atomic_mass=1.66053e-27

kbt=3.16676087e-6 # kbt in hartree 

default_units = {
unit_type_none:'',
unit_type_length:'bohr',
unit_type_energy:'hartree',
unit_type_time:'au_time',
unit_type_longtime:'sec',
unit_type_velocity:'bohr/au_time',
unit_type_force:'hartree/bohr',
unit_type_pressure:'hartree/bohr3',
unit_type_mass:'au_mass',
unit_type_angle:'degree'
}

def get_unit_factor(from_unit, to_unit):
    fr = unit_conversion[from_unit.strip().lower()]
    to = unit_conversion[to_unit.strip().lower()]
    fac=1
    if fr!=None and to!=None:
        fac = to/fr
    return fac

condition  = 'condition'
importance = 'importance'
importance_required='required'
importance_recommended = 'recommended'
importance_optional = 'optional'

begin_tag='[begin]'
end_tag = '[end]'

NOT_READY=1

BLOCK = 'block'
TABLE = 'table'
PRIMITIVE = 'primitive'
VECTOR = 'vector'

import logging

def load_spec_dict(filename):
    ret={}
    try:
        f=open(filename,'r')
        nam=''
        tmpdict={}
        begin=False
        for line in f:
            line=line.strip()
            vals = line.split('=')
            if line.startswith(begin_tag):
                begin=True
            elif line.startswith(end_tag):
                tmpdict={}
            elif len(vals)>=2 and begin:
                if vals[0]==name:
                    ret[vals[1]] = tmpdict
                else:
                    tmpdict[vals[0]]=vals[1]
        f.close()
        return ret
    except IOError:
        logger.error('failed read of: '+filename)
        return None
